@@ -104,10 +104,6 @@ pbgtest.panelmodel <- function(x, order = NULL, type = c("Chisq", "F"), ...) {
 
     ## lmtest::bgtest on the demeaned model:
   
-    ## check package availability and load if necessary ## not needed anymore as importFrom in NAMESPACE is now used
-    #lm.ok <- require("lmtest")
-    #if(!lm.ok) stop("package lmtest is needed but not available")
-  
     ## pbgtest is the return value of lmtest::bgtest, exception made for the method attribute
     auxformula <- demy ~ demX - 1 #if(model == "within") demy~demX-1 else demy~demX
     lm.mod <- lm(auxformula)
@@ -227,22 +223,19 @@ pwtest.formula <- function(x, data, effect = c("individual", "time"), ...) {
 #' @export
 pwtest.panelmodel <- function(x, effect = c("individual", "time"), ...) {
   if (describe(x, "model") != "pooling") stop("pwtest only relevant for pooling models")
-  effect <- match.arg(effect, choices = c("individual", "time")) # was: effect <- describe(x, "effect")
-                                                                 # here we want the effect as in the call of pwtest(),
-                                                                 # not of the already estimated model, because that is
-                                                                 # always a pooling model
+  effect <- match.arg(effect, choices = c("individual", "time"))
   data <- model.frame(x)
   ## extract indices
 
   ## if effect="individual" std., else swap
   index <- attr(data, "index")
   if (effect == "individual"){
-    index <- index[[1]]
-    tindex <- index[[2]]
+    index  <- index[[1L]]
+    tindex <- index[[2L]]
   }
   else{
-    index <- index[[2]]
-    tindex <- index[[1]]
+    index  <- index[[2L]]
+    tindex <- index[[1L]]
   }
   ## det. number of groups and df
   n <- length(unique(index))
@@ -252,7 +245,7 @@ pwtest.panelmodel <- function(x, effect = c("individual", "time"), ...) {
   ## det. total number of obs. (robust vs. unbalanced panels)
   nT <- nrow(X)
   ## det. max. group numerosity
-  t <- max(tapply(X[,1],index,length))
+  t <- max(tapply(X[ , 1L], index, length))
 
   ## ref. Wooldridge (2002), p.264 / Wooldridge (2010), p.299
     
@@ -268,7 +261,7 @@ pwtest.panelmodel <- function(x, effect = c("individual", "time"), ...) {
   ## for each group 1..n 
   ## (possibly different sizes if unbal., thus a list
   ## and thus, unlike Wooldridge (eq.10.37), we divide 
-  ## every block by *his* t(t-1)/2)
+  ## every block by *its* t(t-1)/2)
   unind <- unique(index) # ????
  
   for(i in 1:n) {
@@ -283,7 +276,7 @@ pwtest.panelmodel <- function(x, effect = c("individual", "time"), ...) {
     return(uts)}
   
   ## det. # of upper triangle members (n*t(t-1)/2 if balanced)
-  ti <- sapply(tres, function(x) dim(x)[[1]])
+  ti <- sapply(tres, function(x) dim(x)[[1L]])
   uptrinum <- sum(ti*(ti-1)/2)  # don't need this!!
 
   ## ...apply to list and sum over resulting vector (df corrected)
@@ -378,10 +371,10 @@ pwartest.formula <- function(x, data, ...) {
   cl <- match.call(expand.dots = TRUE)
   if (is.null(cl$model)) cl$model <- "within"
   if (cl$model != "within") stop("pwartest only relevant for within models")
-  if (names(cl)[3] == "") names(cl)[3] <- "data"
-  names(cl)[2] <- "formula"
+  if (names(cl)[3L] == "") names(cl)[3L] <- "data"
+  names(cl)[2L] <- "formula"
   m <- match(plm.arg, names(cl), 0)
-  cl <- cl[c(1L,m)]
+  cl <- cl[c(1L, m)]
   cl[[1L]] <- quote(plm)
   plm.model <- eval(cl, parent.frame())
   pwartest(plm.model, ...)
@@ -403,12 +396,12 @@ pwartest.panelmodel <- function(x, ...) {
   N <- length(FEres)
   FEres.1 <- c(NA,FEres[1:(N-1)])
   index <- attr(data, "index")
-  id <- index[[1]]
-  time <- index[[2]]
-  lagid <- as.numeric(id) - c(NA,as.numeric(id)[1:(N-1)])
-  FEres.1[lagid!=0] <- NA
+  id   <- index[[1L]]
+  time <- index[[2L]]
+  lagid <- as.numeric(id) - c(NA, as.numeric(id)[1:(N-1)])
+  FEres.1[lagid != 0] <- NA
   data <- data.frame(id, time, FEres = unclass(FEres), FEres.1 = unclass(FEres.1))
-  names(data)[c(1,2)] <- c("id", "time")
+  names(data)[c(1L, 2L)] <- c("id", "time")
   data <- na.omit(data)
   
   # calc. auxiliary model
@@ -465,7 +458,7 @@ pwartest.panelmodel <- function(x, ...) {
 #
 ## Implementation follows the formulae for unbalanced panels, which reduce for balanced data to the formulae for balanced panels.
 ##
-## Notation in code largly follows Sosa-Escudero/Bera (2008) (m in Sosa-Escudero/Bera (2008) is total number of observations -> N_obs)
+## Notation in code largely follows Sosa-Escudero/Bera (2008) (m in Sosa-Escudero/Bera (2008) is total number of observations -> N_obs)
 ## NB: Baltagi's book matrix A is slightly different defined: A in Baltagi is -A in Sosa-Escudera/Bera (2008)
 
 
@@ -479,7 +472,7 @@ pwartest.panelmodel <- function(x, ...) {
 #' random effect specification by Baltagi and Li.
 #' 
 #' These Lagrange multiplier tests are robust vs. local
-#' misspecification of the alternative hypothesis, i.e. they test the
+#' misspecification of the alternative hypothesis, i.e., they test the
 #' null of serially uncorrelated residuals against AR(1) residuals in
 #' a pooling model, allowing for local departures from the assumption
 #' of no random effects; or they test the null of no random effects
@@ -602,11 +595,11 @@ pbsytest.formula <- function(x, data, ..., test = c("ar", "re", "j"), re.normal
   cl <- match.call(expand.dots = TRUE)
   if (is.null(cl$model)) cl$model <- "pooling"
   if (cl$model != "pooling") stop("pbsytest only relevant for pooling models")
-  names(cl)[2] <- "formula"
-  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  names(cl)[2L] <- "formula"
+  if (names(cl)[3L] == "") names(cl)[3L] <- "data"
   m <- match(plm.arg ,names(cl), 0)
   cl <- cl[c(1, m)]
-  cl[[1]] <- as.name("plm")
+  cl[[1L]] <- as.name("plm")
   plm.model <- eval(cl, parent.frame())
   pbsytest(plm.model, test = test, re.normal = re.normal, ...)
 }
@@ -625,8 +618,8 @@ pbsytest.panelmodel <- function(x, test = c("ar", "re", "j"), re.normal = if (te
   data <- model.frame(x)
   ## extract indices
   index <- attr(data, "index")
-  tindex <- index[[2]]
-  iindex <- index[[1]]
+  tindex <- index[[2L]]
+  iindex <- index[[1L]]
   
   
   ## till here.
@@ -671,7 +664,7 @@ pbsytest.panelmodel <- function(x, test = c("ar", "re", "j"), re.normal = if (te
   a <- sum(T_i^2) # Sosa-Escudera/Bera (2008), p. 69
   
   switch(test,
-           ar = {
+           "ar" = {
              # RS*_lambda from Sosa-Escudero/Bera (2008), p. 73 (unbalanced formula)
              stat <- (B + (((N_obs - n)/(a - N_obs)) * A))^2 * (((a - N_obs)*N_obs^2) / ((N_obs - n)*(a - 3*N_obs + 2*n)))
              df <- c(df = 1)
@@ -681,8 +674,8 @@ pbsytest.panelmodel <- function(x, test = c("ar", "re", "j"), re.normal = if (te
              myH0_alt <- "AR(1) errors sub random effects"
            },
            
-           re = {
-             if (re.normal) {
+           "re" = {
+             if(re.normal) {
                # RSO*_mu from Sosa-Escudero/Bera (2008), p. 75 (unbalanced formula), normally distributed
                stat <- -sqrt( (N_obs^2) / (2*(a - 3*N_obs + 2*n))) * (A + 2*B)
                names(stat) <- "z"
@@ -701,7 +694,7 @@ pbsytest.panelmodel <- function(x, test = c("ar", "re", "j"), re.normal = if (te
              }
            },
            
-           j = {
+           "j" = {
              # RS_lambda_mu in Sosa-Escudero/Bera (2008), p. 74 (unbalanced formula)
              stat <- N_obs^2 * ( ((A^2 + 4*A*B + 4*B^2) / (2*(a - 3*N_obs + 2*n))) + (B^2/(N_obs - n)))
              # Degrees of freedom in the joint test (test="j") of Baltagi/Li (1991) are 2 (chisquare(2) distributed),
@@ -751,7 +744,7 @@ pbsytest.panelmodel <- function(x, test = c("ar", "re", "j"), re.normal = if (te
 #'     `"formula"`,
 #' @param data a `data.frame`,
 #' @param \dots further arguments to be passed on to `dwtest`,
-#'     e.g.  `alternative`, see [lmtest::dwtest()] for
+#'     e.g., `alternative`, see [lmtest::dwtest()] for
 #'     further details.
 #' @return An object of class `"htest"`.
 #' @export
@@ -850,10 +843,10 @@ pdwtest.formula <- function(x, data, ...) {
 
   cl <- match.call(expand.dots = TRUE)
   if (is.null(cl$model)) cl$model <- "pooling"
-  names(cl)[2] <- "formula"
-  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  names(cl)[2L] <- "formula"
+  if (names(cl)[3L] == "") names(cl)[3L] <- "data"
   m <- match(plm.arg, names(cl), 0)
-  cl <- cl[c(1L,m)]
+  cl <- cl[c(1L, m)]
   cl[[1L]] <- quote(plm)
   plm.model <- eval(cl, parent.frame())
   pdwtest(plm.model, ...)
@@ -912,7 +905,7 @@ pdwtest.formula <- function(x, data, ...) {
 #'     relevant for formula interface),
 #' @param \dots only relevant for formula interface: further arguments
 #'     to specify the model to test (arguments passed on to plm()),
-#'     e.g.  `effect`.
+#'     e.g., `effect`.
 #' @return An object of class `"htest"`.
 #' @export
 #' @author Kevin Tappe
@@ -968,7 +961,7 @@ pbnftest.panelmodel <- function(x, test = c("bnf", "lbi"), ...) {
   # observation is lost per observational unit
   if (!inherits(residuals(x), "pseries")) stop("pdwtest internal error: residuals are not of class \"pseries\"") # check to be safe: need pseries
   
-  ind <- index(x)[[1]]
+  ind <- index(x)[[1L]]
   obs1 <- !duplicated(ind)                  # first ob of each individual
   obsn <- !duplicated(ind, fromLast = TRUE) # last ob of each individual
   
@@ -993,11 +986,11 @@ pbnftest.panelmodel <- function(x, test = c("bnf", "lbi"), ...) {
   }
   
   if (test == "lbi")  {
-    ## d2 contains the "earlier" obs sourrounded by gaps in time periods
+    ## d2 contains the "earlier" obs surrounded by gaps in time periods
     d2_contrib <- as.logical(is.na(lead(residuals(x), shift = "time")) - obsn)
     d2 <- sum(residuals(x)[d2_contrib]^2) / res_crossprod
     
-    ## d3, d4: sum squared residual of first/last time period for all inviduals / crossprod(residuals)
+    ## d3, d4: sum squared residual of first/last time period for all individuals / crossprod(residuals)
     d3 <- sum(residuals(x)[obs1]^2) / res_crossprod
     d4 <- sum(residuals(x)[obsn]^2) / res_crossprod
     
@@ -1026,10 +1019,10 @@ pbnftest.formula <- function(x, data, test = c("bnf", "lbi"), model = c("pooling
   
   cl <- match.call(expand.dots = TRUE)
   if (is.null(model)) model <- "pooling"
-  names(cl)[2] <- "formula"
-  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  names(cl)[2L] <- "formula"
+  if (names(cl)[3L] == "") names(cl)[3L] <- "data"
   m <- match(plm.arg, names(cl), 0)
-  cl <- cl[c(1L,m)]
+  cl <- cl[c(1L, m)]
   cl[[1L]] <- quote(plm)
   plm.model <- eval(cl, parent.frame())
   pbnftest(plm.model, test = test)
@@ -1054,7 +1047,7 @@ pbnftest.formula <- function(x, data, test = c("bnf", "lbi"), model = c("pooling
 #' effects.
 #' 
 #' This is a Lagrange multiplier test for the null of no serial
-#' correlation, against the alternative of either an AR(1) or an MA(1)
+#' correlation, against the alternative of either an AR(1) or a MA(1)
 #' process, in the idiosyncratic component of the error term in a
 #' random effects panel model (as the analytical expression of the
 #' test turns out to be the same under both alternatives,
@@ -1119,7 +1112,7 @@ pbltest.formula <- function(x, data, alternative = c("twosided", "onesided"), in
         data <- pdata.frame(data, index = index)
 
   ## need name of individual index
-  gindex <- dimnames(attr(data, "index"))[[2]][1]
+  gindex <- dimnames(attr(data, "index"))[[2L]][1L]
 
  ## make random effects formula
   rformula <- NULL
@@ -1136,12 +1129,12 @@ pbltest.formula <- function(x, data, alternative = c("twosided", "onesided"), in
   ## make 'bidiagonal' matrix (see BL, p.136)
   G <- matrix(0, ncol = t., nrow = t.)
   for(i in 2:t.) {
-    G[i-1,i] <- 1
-    G[i,i-1] <- 1
-    }
+    G[i-1, i] <- 1
+    G[i, i-1] <- 1
+  }
 
   ## retrieve composite (=lowest level) residuals
-  uhat <- residuals(mymod, level=0)
+  uhat <- residuals(mymod, level = 0)
 
   ## sigma2.e and sigma2.1 as in BL
   ## break up residuals by group to get rid of Kronecker prod.
@@ -1182,15 +1175,15 @@ pbltest.formula <- function(x, data, alternative = c("twosided", "onesided"), in
 
   ## build up information matrix
   Jmat <- matrix(nrow = 3, ncol = 3)
-  Jmat[1,] <- c(j.rr, j.12, j.13)
-  Jmat[2,] <- c(j.12, j.22, j.23)
-  Jmat[3,] <- c(j.13, j.23, j.33)
+  Jmat[1, ] <- c(j.rr, j.12, j.13)
+  Jmat[2, ] <- c(j.12, j.22, j.23)
+  Jmat[3, ] <- c(j.13, j.23, j.33)
 
   J11 <- n.^2 * t.^2 * (t.-1) / (det(Jmat) * 4*sigma2.1^2 * sigma2.e^2)
   ## this is the same as J11 <- solve(Jmat)[1,1], see BL page 73
 
   switch(match.arg(alternative),
-         onesided = {
+         "onesided" = {
            LMr.m <- Drho * sqrt(J11)
            pval <- pnorm(LMr.m, lower.tail = FALSE)
            names(LMr.m) <- "z"
@@ -1198,7 +1191,7 @@ pbltest.formula <- function(x, data, alternative = c("twosided", "onesided"), in
            method2 <- "H0: rho = 0, HA: rho > 0"
            parameter <- NULL
          },
-         twosided = {
+         "twosided" = {
            LMr.m <- Drho^2 * J11
            pval <- pchisq(LMr.m, df = 1, lower.tail = FALSE)
            names(LMr.m) <- "chisq"
@@ -1229,7 +1222,8 @@ pbltest.plm <- function(x, alternative = c("twosided", "onesided"), ...) {
   if (describe(x, "model") != "random") stop("Test is only for random effects models.")
   
   # call pbltest.formula the right way
-  pbltest.formula(formula(x$formula), data=cbind(index(x), x$model), index=names(index(x)), alternative = alternative, ...)
+  pbltest.formula(formula(x$formula), data=cbind(index(x), x$model),
+                  index=names(index(x)), alternative = alternative, ...)
 }
 
 #' Wooldridge first--difference--based test for AR(1) errors in levels
@@ -1327,10 +1321,10 @@ pwfdtest <- function(x, ...) {
 pwfdtest.formula <- function(x, data, ..., h0 = c("fd", "fe")) {
   cl <- match.call(expand.dots = TRUE)
   if (is.null(cl$model)) cl$model <- "fd"
-  names(cl)[2] <- "formula"
-  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  names(cl)[2L] <- "formula"
+  if (names(cl)[3L] == "") names(cl)[3L] <- "data"
   m <- match(plm.arg, names(cl), 0)
-  cl <- cl[c(1L,m)]
+  cl <- cl[c(1L, m)]
   cl[[1L]] <- quote(plm)
   plm.model <- eval(cl, parent.frame())
   pwfdtest(plm.model, ..., h0 = h0)
@@ -1352,8 +1346,8 @@ pwfdtest.panelmodel <- function(x, ..., h0 = c("fd", "fe")) {
   ## this is an ad-hoc solution for the fact that the 'fd' model
   ## carries on the full indices while losing the first time period
   index <- attr(model.frame(x), "index")
-  time <- as.numeric(index[[2]])
-  id <- as.numeric(index[[1]])
+  time <- as.numeric(index[[2L]])
+  id   <- as.numeric(index[[1L]])
   
   ## fetch dimensions and adapt to those of indices
   pdim <- pdim(x)
@@ -1361,7 +1355,7 @@ pwfdtest.panelmodel <- function(x, ..., h0 = c("fd", "fe")) {
   Ti_minus_one <- pdim$Tint$Ti-1
   
   ## generate new individual index: drop one observation per individual
-  ## NB: This is based on the assumption that the estimated FD model performes
+  ## NB: This is based on the assumption that the estimated FD model performs
   ##     its diff-ing row-wise (it currently does so). If the diff-ing for FD
   ##     is changed to diff-ing based on time dimension, this part about index
   ##     creation needs to be re-worked because more than 1 observation per
@@ -1375,24 +1369,25 @@ pwfdtest.panelmodel <- function(x, ..., h0 = c("fd", "fe")) {
   if(length(red_id) == 0)
     stop("only individuals with one observation in original data: test not feasible")
   
-  # make pdata.frame for auxiliary regression: time dimension is not relvant
+  # make pdata.frame for auxiliary regression: time dimension is not relevant
   # as the first observation of each individual was dropped -> let time dimension
   # be created (is not related to the original times anymore)
   auxdata <- pdata.frame(as.data.frame(cbind(red_id, FDres)), index = "red_id")
   
-  # lag residuals by row (as the FD model diffes by row)
-  # NB: need to consoder change to shift = "time" if behaviour of FD model is changed
+  # lag residuals by row (as the FD model diffs by row)
+  # NB: need to consider change to shift = "time" if behaviour of FD model is changed
   auxdata[["FDres.1"]] <- lag(auxdata[["FDres"]], shift = "row")
   
   ## pooling model FDres vs. lag(FDres), with intercept (might as well do it w.o.)
   auxmod <- plm(FDres ~ FDres.1, data = auxdata, model = "pooling")
   
   switch(match.arg(h0), 
-         fd = {h0des <- "differenced"
+         "fd" = {h0des <- "differenced"
          ## theoretical rho under H0: no serial 
          ## corr. in differenced errors is 0
          rho.H0 <- 0},
-         fe = {h0des <- "original"
+         
+         "fe" = {h0des <- "original"
          ## theoretical rho under H0: no serial 
          ## corr. in original errors is -0.5
          rho.H0 <- -0.5})
